@@ -1,6 +1,6 @@
 # The MIT License (MIT)
 #
-# Copyright (c) 2016 Philippe Proulx <eepp.ca>
+# Copyright (c) 2016-2017 Philippe Proulx <eepp.ca>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,10 +23,13 @@
 from collections import namedtuple
 from vlttng.utils import perror
 from termcolor import colored
+import pkg_resources
 import argparse
 import platform
 import vlttng
+import shlex
 import enum
+import os
 import re
 
 
@@ -108,98 +111,48 @@ class _Wizard:
         }
         self._project_name_to_title = {
             'babeltrace': ('Babeltrace',),
-            'elfutils': ('elfutils', 'dependency of Babeltrace'),
+            'elfutils': ('elfutils', 'optional dependency of Babeltrace'),
             'glib': ('GLib', 'dependency of Babeltrace'),
             'libxml2': ('libxml2', 'dependency of LTTng-tools'),
             'lttng-analyses': ('LTTng analyses',),
             'lttng-modules': ('LTTng-modules',),
             'lttng-tools': ('LTTng-tools',),
             'lttng-ust': ('LTTng-UST',),
+            'popt': ('popt', 'dependency of Babeltrace and LTTng-tools'),
             'tracecompass': ('Trace Compass',),
-            'urcu': ('Userspace RCU',),
+            'lttng-scope': ('LTTng Scope',),
+            'urcu': ('Userspace RCU', 'dependency of LTTng-tools and LTTng-UST'),
         }
         self._project_name_to_versions = {
-            'babeltrace': (
-                'stable-1.2',
-                'stable-1.3',
-                'stable-1.4',
-            ),
-            'glib': (
-                '2.22.5',
-                '2.23.6',
-                '2.24.2',
-                '2.25.17',
-                '2.26.1',
-                '2.27.93',
-                '2.28.8',
-                '2.29.92',
-                '2.30.3',
-                '2.31.22',
-                '2.32.4',
-                '2.33.14',
-                '2.34.3',
-                '2.35.9',
-                '2.36.4',
-                '2.37.93',
-                '2.38.2',
-                '2.39.92',
-                '2.40.2',
-                '2.41.5',
-                '2.42.2',
-                '2.43.92',
-                '2.44.1',
-                '2.45.8',
-                '2.46.2',
-                '2.47.92',
-                '2.48.1',
-            ),
-            'libxml2': (
-                '2.8.0',
-                '2.9.0',
-                '2.9.1',
-                '2.9.2',
-                '2.9.3',
-            ),
-            'lttng-analyses': (
-                '0.3.0',
-                '0.4.0',
-                '0.4.1',
-                '0.4.2',
-                '0.4.3',
-                '0.5.0',
-                '0.5.1',
-                '0.5.2',
-                '0.5.3',
-                '0.5.4',
-            ),
-            'lttng-modules': (
-                'stable-2.6',
-                'stable-2.7',
-                'stable-2.8',
-            ),
-            'lttng-tools': (
-                'stable-2.6',
-                'stable-2.7',
-                'stable-2.8',
-            ),
-            'lttng-ust': (
-                'stable-2.6',
-                'stable-2.7',
-                'stable-2.8',
-            ),
-            'tracecompass': (
-                '1.1.0',
-                '1.2.0',
-                '1.2.1',
-                '2.0.0',
-            ),
-            'urcu': (
-                'stable-0.7',
-                'stable-0.8',
-                'stable-0.9',
-            ),
+            'babeltrace': [],
+            'elfutils': [],
+            'glib': [],
+            'libxml2': [],
+            'lttng-analyses': [],
+            'lttng-modules': [],
+            'lttng-tools': [],
+            'lttng-ust': [],
+            'popt': [],
+            'tracecompass': [],
+            'lttng-scope': [],
+            'urcu': [],
         }
 
+        dirname = pkg_resources.resource_filename(__name__,
+                                                  vlttng._PROFILES_DIRNAME)
+
+        for filename in sorted(os.listdir(dirname)):
+            if filename.endswith('.yml'):
+                profile = filename[:-4]
+
+                if not re.match(r'^.+-\d+(\.\d+)*$', profile):
+                    continue
+
+                for project_name in self._project_name_to_versions:
+                    if profile.startswith(project_name):
+                        profile_suffix = profile[len(project_name) + 1:]
+                        self._project_name_to_versions[project_name].append(profile_suffix)
+
     def _handle_state(self):
         self._state_handlers[self._state]()
 
@@ -214,7 +167,7 @@ class _Wizard:
             args += ['-o', 'build-env.PYTHON={}'.format(self._python_interpreter)]
             args += ['-o', 'build-env.PYTHON_CONFIG={}-config'.format(self._python_interpreter)]
 
-        args.append(self._path)
+        args.append(shlex.quote(self._path))
 
         return args
 
@@ -224,7 +177,7 @@ class _Wizard:
 
 This questionnaire will help you create a basic vlttng profile to build
 the projects and features you need.
-'''.format(_bold('..')))
+''')
 
         while self._state != _WizardState.END:
             self._handle_state()
@@ -247,10 +200,18 @@ the projects and features you need.
             print(_cquestion('How many make jobs would you like?'))
             print()
 
+            import multiprocessing
+
+            prompt_ext = ' [{}]'.format(multiprocessing.cpu_count())
+
             while True:
-                answer = self._input(suffix='')
+                answer = self._input(prompt_ext, '')
                 print()
 
+                if answer == '':
+                    ianswer = multiprocessing.cpu_count()
+                    break
+
                 try:
                     ianswer = int(answer.strip())
                 except:
@@ -282,7 +243,9 @@ the projects and features you need.
             self._get_project_title('lttng-modules'),
             self._get_project_title('lttng-tools'),
             self._get_project_title('lttng-ust'),
+            self._get_project_title('popt'),
             self._get_project_title('tracecompass'),
+            self._get_project_title('lttng-scope'),
             self._get_project_title('urcu'),
         )
         choice_projects = (
@@ -294,7 +257,9 @@ the projects and features you need.
             'lttng-modules',
             'lttng-tools',
             'lttng-ust',
+            'popt',
             'tracecompass',
+            'lttng-scope',
             'urcu',
         )
         self._pchoices(choices)
@@ -302,7 +267,7 @@ the projects and features you need.
         choices = self._get_choices(len(choices))
         print()
 
-        if choices == -1:
+        if choices == 'a':
             self._projects = [project for project in choice_projects]
         else:
             for choice in choices:
@@ -317,24 +282,25 @@ the projects and features you need.
         choice_projects = []
 
         for project in self._projects:
-            if project != 'elfutils':
+            if project not in ('elfutils', 'popt'):
                 choices.append(self._get_project_title(project))
                 choice_projects.append(project)
 
+        if len(choice_projects) == 0:
+            self._state = _WizardState.ASK_VERSIONS
+            return
+
         self._pchoices(choices)
-        self._pmultiple_choices_info()
-        choices = self._get_choices(len(choices))
+        self._pmultiple_choices_info(with_none=True)
+        choices = self._get_choices(len(choices), True)
         print()
 
-        if choices == -1:
+        if choices == 'a':
             self._projects_versions = {p: 'master' for p in choice_projects}
         else:
             for choice in choices:
                 self._projects_versions[choice_projects[choice - 1]] = 'master'
 
-        if 'elfutils' in self._projects:
-            self._projects_versions['elfutils'] = '0.166'
-
         self._state = _WizardState.ASK_VERSIONS
 
     def _state_ask_versions(self):
@@ -492,9 +458,16 @@ the projects and features you need.
     def _get_project_title(self, name):
         return self._project_name_to_title[name]
 
-    def _pmultiple_choices_info(self):
+    def _pmultiple_choices_info(self, with_none=False):
         print()
-        print('Enter a space-delimited list of choices, or {} to select all.'.format(_bold('a')))
+        print('Enter a space-delimited list of choices,', end='')
+
+        if with_none:
+            print(' {} to select all,'.format(_bold('a')))
+            print('or {} to select none.'.format(_bold('n')))
+        else:
+            print(' or {} to select all.'.format(_bold('a')))
+
         print()
 
     def _pchoices(self, choices):
@@ -535,19 +508,30 @@ the projects and features you need.
 
         return choice
 
-    def _try_get_choices(self, choice_max):
-        resp = self._input()
+    def _try_get_choices(self, choice_max, all_default):
+        prompt_ext = ''
+
+        if all_default:
+            prompt_ext = ' ({} for none) [{}]'.format(_bold('n'), _bold('a'))
+
+        resp = self._input(prompt_ext)
+
+        if resp == '':
+            if all_default:
+                return 'a'
 
-        if len(resp) == 0:
             self._pinvalid_choices()
             return
 
+        if resp == 'n':
+            return set()
+
         raw_choices = re.split(r'\s+', resp)
         choices = set()
 
         for raw_choice in raw_choices:
             if raw_choice == 'a':
-                return -1
+                return 'a'
 
             choice = self._raw_choice_to_int(raw_choice, choice_max)
 
@@ -559,9 +543,9 @@ the projects and features you need.
 
         return sorted(list(choices))
 
-    def _get_choices(self, choice_max):
+    def _get_choices(self, choice_max, all_default=False):
         while True:
-            choices = self._try_get_choices(choice_max)
+            choices = self._try_get_choices(choice_max, all_default)
 
             if choices is not None:
                 return choices
@@ -569,7 +553,7 @@ the projects and features you need.
     def _try_get_choice(self, choice_max):
         resp = self._input(suffix='')
 
-        if len(resp) == 0:
+        if resp == '':
             self._pinvalid_choices(None, '')
             return
 
